##
# File:    DictionaryRegistry.py
# Author:  jdw
# Date:    3-Oct-2013
# Version: 0.001
#
# Updates:
#     7-Jul-2018  ep  mmcif_ma and remove generation of mmcif_mdb to registry
##
"""
Classes providing a registry of essential information about known data dictionaries and data item coverage statistics.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"

import logging

from rcsb.utils.io.MarshalUtil import MarshalUtil

logger = logging.getLogger(__name__)


class DictionaryRegistry(object):
    def __init__(self, registryPath):
        rObj = self.__getRegistry(registryPath)
        self.__otherDictionaryNameList = rObj["otherDictionaryNameList"]
        self.__internalDictionaryNameList = rObj["internalDictionaryNameList"]
        self.__pdbxDictionaryNameList = rObj["pdbxDictionaryNameList"]
        self.__dictionaryNameList = []
        self.__dictionaryNameList.extend(self.__pdbxDictionaryNameList)
        self.__dictionaryNameList.extend(self.__otherDictionaryNameList)
        self.__dictInfoD = rObj["dictionaryInfo"]

    def __getRegistry(self, registryPath):
        """"""
        try:
            mU = MarshalUtil()
            obj = mU.doImport(registryPath, fmt="json")
            return obj["mmcif_dictionary_registry"]
        except Exception as e:
            logger.exception("Failing for %r with %s", registryPath, str(e))

    def getDictionaryNameList(self):
        return self.__dictionaryNameList

    def getInternalDictionaryNameList(self):
        return self.__internalDictionaryNameList

    def getPdbxDictionaryNameList(self):
        return self.__pdbxDictionaryNameList

    def getPdbmlSchemaNameList(self):
        oL = ["mmcif_pdbx_v42"]
        oL.extend(self.__dictionaryNameList)
        return oL

    def get(self):
        return self.__dictInfoD

    def getTitle(self, dictionaryName):
        try:
            return self.__dictInfoD[dictionaryName]["title"]
        except Exception as e:
            logger.debug("Failing with %s", str(e))
            return None

    def getDescription(self, dictionaryName):
        try:
            return self.__dictInfoD[dictionaryName]["description"]
        except Exception as e:
            logger.debug("Failing with %s", str(e))
            return None

    def getDevelopers(self, dictionaryName):
        try:
            return self.__dictInfoD[dictionaryName]["developers"]
        except Exception as e:
            logger.debug("Failing with %s", str(e))
            return None

    def getMaintainers(self, dictionaryName):
        try:
            return self.__dictInfoD[dictionaryName]["maintainers"]
        except Exception as e:
            logger.debug("Failing with %s", str(e))
            return None

    def getSchemaName(self, dictionaryName):
        try:
            return self.__dictInfoD[dictionaryName]["schema"]
        except Exception as e:
            logger.debug("Failing with %s", str(e))
            return None



##
# File:    NeighborFigures.py
# Author:  J. Westbrook
# Date:    27-Sep-2013
# Version: 0.001
#
# Updates:
#
#  03-Oct-2013 jdw  -  Add guard characters before category and attribute names in dot files
#                      to protect against leading digits in cif names.
#   8-Oct-2013 jdw -   Adjust cell padding for for attribute name display
#  28-Dec-2020 jdw -   cleanup and py39
##
"""
Utility methods for generating depictions of data category neighbor relationships.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"

import logging
import os

from mmcif.api.PdbxContainers import CifName

logger = logging.getLogger(__name__)


class NeighborFigures(object):
    """Utility methods for generating depictions of data category neighbor relationships"""

    def __init__(self, dictApiObj, pathInfoObj=None, pathDot="/usr/local/bin/dot", verbose=False):
        """"""
        self.__verbose = verbose
        self.__dApi = dictApiObj
        self.__pI = pathInfoObj
        self.__pathDot = pathDot
        # Default font settings ----
        self.__fontFace = "helvetica"
        # self.__fontSize='10'
        self.__fontSizeCategory = "10"
        self.__fontSizeAttribute = "9"
        self.__titleFontFace = "helvetica"
        self.__titleFontSize = "18"
        self.__subTitleFontSize = "14"
        #
        self.__itemCounts = {"archive": {}, "prd": {}, "cc": {}, "family": {}}
        self.__categoryCounts = {"archive": {}, "prd": {}, "cc": {}, "family": {}}

    def setFonts(self, fontFace="helvetica", fontSizeCategory="10", fontSizeAttribute="10", titleFontSize="18", subTitleFontSize="14"):
        self.__fontFace = fontFace
        self.__fontSizeCategory = fontSizeCategory
        self.__fontSizeAttribute = fontSizeAttribute
        self.__titleFontFace = fontFace
        self.__titleFontSize = titleFontSize
        self.__subTitleFontSize = subTitleFontSize

    def setItemCounts(self, itemNameD, deliveryType="archive"):
        for itemName, itemCount in itemNameD.items():
            self.__itemCounts[deliveryType][itemName] = itemCount
            categoryName = CifName.categoryPart(itemName)
            if categoryName not in self.__categoryCounts[deliveryType]:
                self.__categoryCounts[deliveryType][categoryName] = itemCount
            else:
                self.__categoryCounts[deliveryType][categoryName] = max(itemCount, self.__categoryCounts[deliveryType][categoryName])

        logger.debug("+NeighborFigures.setItemCounts() items      in archive count %d", len(self.__itemCounts[deliveryType]))
        logger.debug("+NeighborFigures.setItemCounts() categories in archive count %d", len(self.__categoryCounts[deliveryType]))

    def getCategoryUseCount(self, categoryName, deliveryType="archive"):
        if categoryName in self.__categoryCounts[deliveryType]:
            return self.__categoryCounts[deliveryType][categoryName]
        else:
            return 0

    def __isCategoryUsed(self, categoryName, deliveryType="archive"):
        if categoryName in self.__categoryCounts[deliveryType]:
            return True
        else:
            return False

    def __isItemUsed(self, itemName, deliveryType="archive"):
        if self.__getItemCount(itemName, deliveryType) > 0:
            return True
        else:
            return False

    def __getItemCount(self, itemName, deliveryType="archive"):
        if itemName in self.__itemCounts[deliveryType]:
            return self.__itemCounts[deliveryType][itemName]
        else:
            return 0

    def __getCategoryUsePercent(self, categoryName, deliveryType="archive"):
        if categoryName in self.__categoryCounts[deliveryType]:
            denom = self.__itemCounts[deliveryType]["_entry.id"]
            pc = 100.0 * float(self.__categoryCounts[deliveryType][categoryName]) / float(denom)

            if pc > 1:
                return "%5d" % (pc)
            elif pc > 0.10:
                return "%5.1f" % (pc)
            elif pc > 0.01:
                return "%5.2f" % (pc)
            else:
                return "%6.3f" % (pc)
        else:
            return "0.0"

    def __getItemUsePercent(self, itemName, deliveryType="archive"):
        if itemName in self.__itemCounts[deliveryType]:
            denom = self.__itemCounts[deliveryType]["_entry.id"]
            pc = 100.0 * float(self.__itemCounts[deliveryType][itemName]) / float(denom)

            if pc > 1:
                return "%5d" % (pc)
            elif pc > 0.10:
                return "%5.1f" % (pc)
            elif pc > 0.01:
                return "%5.2f" % (pc)
            else:
                return "%6.3f" % (pc)
        else:
            return "0.0"

    def __assignItemIconType(self, itemNameList):
        iconTypeList = []
        #
        categoryName = CifName.categoryPart(itemNameList[0])
        keyItemNameList = self.__dApi.getCategoryKeyList(categoryName)
        for itemName in itemNameList:
            tType = "none"
            attributeName = CifName.attributePart(itemName)
            aMan = self.__dApi.getMandatoryCode(categoryName, attributeName) in ["yes", "y"]
            dMan = self.__dApi.getMandatoryCodeAlt(categoryName, attributeName, fallBack=False) in ["yes", "y"]
            inArchive = self.__getItemCount(itemName, deliveryType="archive") > 0
            inChemDict = self.__getItemCount(itemName, deliveryType="cc") > 0
            inBirdDict = self.__getItemCount(itemName, deliveryType="prd") > 0 or self.__getItemCount(itemName, deliveryType="family") > 0

            isKey = itemName in keyItemNameList
            if isKey:
                tType = "key"
            elif aMan and dMan:
                tType = "all-mandatory"
            elif aMan:
                tType = "mandatory"
            elif dMan:
                tType = "deposit-mandatory"
            if inArchive:
                tType += "+database"
            if inChemDict:
                tType += "+chem-dict"
            if inBirdDict:
                tType += "+bird-dict"
            iconTypeList.append(tType)

        return iconTypeList

    def __assignCategoryIconType(self, categoryNameList):
        iconTypeList = []
        #
        for categoryName in categoryNameList:
            tType = "none"
            cMan = self.__dApi.getCategoryMandatoryCode(categoryName) in ["yes", "y"]
            inArchive = self.__isCategoryUsed(categoryName, deliveryType="archive")
            inChemDict = self.__isCategoryUsed(categoryName, deliveryType="cc")
            inBirdDict = self.__isCategoryUsed(categoryName, deliveryType="prd") or self.__isCategoryUsed(categoryName, deliveryType="family")
            if cMan:
                tType = "mandatory"
            if inArchive:
                tType += "+database"
            if inChemDict:
                tType += "+chem-dict"
            if inBirdDict:
                tType += "+bird-dict"

            iconTypeList.append(tType)

        return iconTypeList

    def __getOrderedItemNameList(self, categoryName, fkList=None, filterDelivery=False, deliveryType="archive"):
        """Return the item name list for the input category with the order: key items, items with FK relationships,
        and remaining items.    Items which do are not key nor participate in any parent child relationships
        can be filtered according to their presence in current archive entries.
        """
        allItemList = sorted(self.__dApi.getItemNameList(categoryName))
        rList = []
        for itemName in allItemList:
            if fkList and itemName in fkList:
                rList.append(itemName)

        tList = []
        keyItemNameList = sorted(self.__dApi.getCategoryKeyList(categoryName))
        if filterDelivery:
            for itemName in allItemList:
                if itemName in keyItemNameList:
                    tList.append(itemName)
                    continue
                if itemName in rList:
                    tList.append(itemName)
                    continue
                if self.__isItemUsed(itemName, deliveryType=deliveryType):
                    tList.append(itemName)
        else:
            tList = allItemList

        #
        for keyItemName in keyItemNameList:
            if keyItemName in rList:
                rList.remove(keyItemName)
            if keyItemName in tList:
                tList.remove(keyItemName)
            else:
                logger.warning("+NeighborFigures.__getOrderedItemNameList() Category %s is missing key definition for %s", categoryName, keyItemName)

        for rItemName in rList:
            if rItemName in tList:
                tList.remove(rItemName)
        #
        minItemCount = len(keyItemNameList) + len(rList)
        #
        itemNameList = []
        itemNameList.extend(keyItemNameList)
        itemNameList.extend(rList)
        itemNameList.extend(tList)
        return itemNameList, minItemCount

    def __getParentCategories(self, itemNameList):
        parentCategories = set()
        for itemName in itemNameList:
            categoryName = CifName.categoryPart(itemName)
            attributeName = CifName.attributePart(itemName)
            parentItemList = self.__dApi.getFullParentList(categoryName, attributeName)
            for parentItem in parentItemList:
                parentCategoryName = CifName.categoryPart(parentItem)
                parentCategories.add(parentCategoryName)
        return list(parentCategories)

    def __getChildCategories(self, itemNameList):
        childCategories = set()
        for itemName in itemNameList:
            categoryName = CifName.categoryPart(itemName)
            attributeName = CifName.attributePart(itemName)
            childItemList = self.__dApi.getFullChildList(categoryName, attributeName)
            for childItem in childItemList:
                childCategoryName = CifName.categoryPart(childItem)
                childCategories.add(childCategoryName)
        return list(childCategories)

    def __getRelativesAdjacent(self, itemNameList):
        aR = {}
        for itemName in itemNameList:
            categoryName = CifName.categoryPart(itemName)
            attributeName = CifName.attributePart(itemName)
            tD = {}
            tD["parentItems"] = self.__dApi.getFullParentList(categoryName, attributeName)
            tD["childItems"] = self.__dApi.getFullChildList(categoryName, attributeName)
            aR[itemName] = tD
        # if (self.__verbose):
        #    for k,v in aR.items():
        #        logger.debug("Item %s\n       parents: %s\n       children  %s\n\n" % (k,v['parentItems'],v['childItems']))
        return aR

    def __renderCategory(self, categoryName, fkList=None, highLight="adjacent", maxItems=10, filterDelivery=False, deliveryType="archive"):
        """Create graphviz object with embedded tabular HTML represenation of the input data category

             * Add _ guard characters before graph name and port names --

        _categoryName_1 [label=<
        <table border="0" cellborder="1" cellspacing="0" align="left">
        <tr><td BGCOLOR="Lavender">categoryName_1</td></tr>
        <tr><td PORT="__attribute_1">attribute_1</td></tr>
        <tr><td PORT="__attribute_2">attribute_2</td></tr>
        <tr><td PORT="__attribute_3">attribute_3</td></tr>
        </table>>];

        """
        colorD = {
            "op1": 'BGCOLOR="#7fc97f"',
            "other": 'BGCOLOR="#beaed4"',
            "op2": 'BGCOLOR="#fdc086"',
            "key": 'BGCOLOR="#ffff99"',
            "op4": 'BGCOLOR="#386cb0"',
            "current": 'BGCOLOR="#f0027f"',
            "adjacent": 'BGCOLOR="#99c49b"',
        }
        oList = []
        itemNameList, minItemCount = self.__getOrderedItemNameList(categoryName, fkList=fkList, filterDelivery=filterDelivery, deliveryType=deliveryType)
        itemsToRender = max(minItemCount, maxItems)
        #
        categoryUrl = os.path.join(self.__pI.getContentTypeObjUrl(contentObjName=categoryName, contentType="Categories"))
        logger.debug("Rendering %s categoryUrl %r itemNameList %r itemsToRender %r fkList %r", categoryName, categoryUrl, itemNameList, itemsToRender, fkList)
        if len(itemNameList) > 0:
            iconTypeList = self.__assignItemIconType(itemNameList)
            oList.append('_%s [label=<<TABLE BORDER="0" CELLBORDER="1" CELLSPACING="0" ALIGN="LEFT">' % (categoryName))
            tdText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__fontSizeCategory, self.__fontFace, categoryName.upper())
            oList.append('    <tr><td %s CELLPADDING="4" HREF="%s" TARGET="_top">%s</td></tr>' % (colorD[highLight], categoryUrl, tdText))

            for iconType, itemName in list(zip(iconTypeList, itemNameList))[:itemsToRender]:
                itemUrl = os.path.join(self.__pI.getContentTypeObjUrl(contentObjName=itemName, contentType="Items"))
                attributeName = CifName.attributePart(itemName)
                tdText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__fontSizeAttribute, self.__fontFace, attributeName)
                if "key" in iconType:
                    oList.append('<tr><td %s PORT="__%s" CELLPADDING="4" HREF="%s" TARGET="_top" ALIGN="LEFT">%s</td></tr>' % (colorD["key"], attributeName, itemUrl, tdText))
                else:
                    oList.append('<tr><td PORT="__%s" CELLPADDING="4" HREF="%s" TARGET="_top" ALIGN="LEFT">%s</td></tr>' % (attributeName, itemUrl, tdText))

            if len(itemNameList) > itemsToRender:
                tdText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__fontSizeAttribute, self.__fontFace, "... and others ...")
                oList.append("<tr><td>%s</td></tr>" % tdText)
            oList.append("</TABLE>>];")
        else:
            # Placeholder for missing category
            oList.append('_%s [label=<<TABLE BORDER="0" CELLBORDER="1" CELLSPACING="0" ALIGN="LEFT">' % (categoryName))
            tdText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__fontSizeCategory, self.__fontFace, categoryName.upper())
            oList.append('    <tr><td %s CELLPADDING="4"  TARGET="_top">%s</td></tr>' % (colorD[highLight], tdText))

            for itemName in fkList:
                iconType = "none"
                itemUrl = os.path.join(self.__pI.getContentTypeObjUrl(contentObjName=itemName, contentType="Items"))
                attributeName = CifName.attributePart(itemName)
                tdText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__fontSizeAttribute, self.__fontFace, attributeName)
                if "key" in iconType:
                    oList.append('<tr><td %s PORT="__%s" CELLPADDING="4" TARGET="_top" ALIGN="LEFT">%s</td></tr>' % (colorD["key"], attributeName, tdText))
                else:
                    oList.append('<tr><td PORT="__%s" CELLPADDING="4" TARGET="_top" ALIGN="LEFT">%s</td></tr>' % (attributeName, tdText))

            oList.append("</TABLE>>];")
        #
        return oList

    def __getRelatedList(self, categoryName, adjacentD):
        #
        relatedSet = set()
        for itemName, adjD in adjacentD.items():
            cName = CifName.categoryPart(itemName)
            if len(adjD["parentItems"]) > 0:
                if cName == categoryName:
                    relatedSet.add(itemName)
                for parentItemName in adjD["parentItems"]:
                    pName = CifName.categoryPart(parentItemName)
                    if pName == categoryName:
                        relatedSet.add(parentItemName)
            if len(adjD["childItems"]) > 0:
                if cName == categoryName:
                    relatedSet.add(itemName)
                for childItemName in adjD["childItems"]:
                    chName = CifName.categoryPart(childItemName)
                    if chName == categoryName:
                        relatedSet.add(childItemName)
        relatedList = sorted(list(relatedSet))

        logger.debug("%s items with parent/child relationships %s", categoryName, len(relatedList))
        #
        return relatedList

    def __generateDotInstructions(
        self,
        categoryName,
        graphTitle=None,
        graphSubTitle=None,
        titleFormat="text",
        maxItems=20,
        filterDelivery=False,
        deliveryType="archive",
        neighborCategoryList=None,
        maxCategories=None,
    ):
        """Internal method producing GraphViz 'dot' instructions depicting data category relationships between the input category and either all
        of its adjacent neighbors or for selected 'neighborCategoryList'.    Optionally apply filtering to categories in current use within the archive.

        maxItems       controls target maximum number of attributes in any category object depiction.
        maxCategories  limits the number of related category objects depicted.
        """
        #
        # Skip cases where the principal category is not associated with the input delivery type -
        #
        if filterDelivery and not self.__isCategoryUsed(categoryName=categoryName, deliveryType=deliveryType):
            logger.debug("skipping category %r delivery %r", categoryName, deliveryType)
            return [], 0
        #
        numCategoriesRendered = 0
        itemNameList = self.__dApi.getItemNameList(categoryName)
        aR = self.__getRelativesAdjacent(itemNameList)

        #
        for k, v in aR.items():
            logger.debug("%s relatives %s", k, v)
        #
        adjacentCategories = []
        adjacentCategories.append(categoryName)
        if neighborCategoryList is None:
            adjacentCategories.extend(self.__getParentCategories(itemNameList))
            adjacentCategories.extend(self.__getChildCategories(itemNameList))
        else:
            adjacentCategories.extend(neighborCategoryList)
            #
        adjacentCategories = sorted(list(set(adjacentCategories)))

        if maxCategories is not None:
            adjacentCategories = adjacentCategories[:maxCategories]

        logger.debug("adjacent categories %s", adjacentCategories)
        #
        oL = []
        oL.append("digraph %s {" % categoryName)
        #  Some previous layout parameters --- jdw
        #        oL.append('splines=true; overlap=portho; model=subset;')
        #        oL.append('splines=ortho; overlap=compress; model=subset; ratio=1.0;')
        oL.append("splines=true; overlap=compress; ")
        #
        # Option graph title -
        if graphTitle is not None:
            if titleFormat == "text":
                # dot instructions do not recognize the font settings.
                # oL.append('graph [labelloc=b, labeljust=left, labelfontname=Helvetica, labelfontsize=18, label="%s"];' % (self.__titleFontFace, self.__titleFontSize, graphTitle))
                oL.append('graph [labelloc=b, labeljust=left, labelfontname=%s, labelfontsize=%s, label="%s"];' % (self.__titleFontFace, self.__titleFontSize, graphTitle))
            else:
                # Title is rendered with this font detail.

                titleText = '<FONT POINT-SIZE="%s" FACE="%s">%s</FONT>' % (self.__titleFontSize, self.__titleFontFace, graphTitle)
                if graphSubTitle is not None:
                    titleText += '<FONT POINT-SIZE="%s" FACE="%s"> <br/> %s</FONT>' % (self.__subTitleFontSize, self.__titleFontFace, graphSubTitle)
                oL.append("graph [labelloc=b, label=<%s>];" % (titleText))

        oL.append("node [shape=plaintext]")

        for catName in adjacentCategories:
            if filterDelivery and not self.__isCategoryUsed(categoryName=catName, deliveryType=deliveryType):
                continue
            if catName == categoryName:
                highLight = "current"
            else:
                highLight = "adjacent"
            relatedList = self.__getRelatedList(categoryName=catName, adjacentD=aR)
            oL.extend(self.__renderCategory(catName, fkList=relatedList, highLight=highLight, maxItems=maxItems, filterDelivery=filterDelivery, deliveryType=deliveryType))
            numCategoriesRendered += 1

        #  --------
        # JDW regenerate full item list -
        #
        itemNameList = []
        for catName in adjacentCategories:
            itemNameList.extend(self.__dApi.getItemNameList(catName))
        aR = {}
        aR = self.__getRelativesAdjacent(itemNameList)
        #
        # --------
        lD = {}
        #
        for itemName in itemNameList:
            if filterDelivery and not self.__isItemUsed(itemName=itemName, deliveryType=deliveryType):
                continue
            tD = aR[itemName]
            for parentItemName in tD["parentItems"]:
                if filterDelivery and not self.__isItemUsed(itemName=parentItemName, deliveryType=deliveryType):
                    continue
                catName = CifName.categoryPart(itemName)
                attName = CifName.attributePart(itemName)
                catParent = CifName.categoryPart(parentItemName)
                attParent = CifName.attributePart(parentItemName)
                #
                if (itemName, parentItemName) in lD:
                    continue
                if catParent not in adjacentCategories:
                    continue
                if catParent != catName:
                    oL.append(" _%s:__%s:w -> _%s:__%s:w;" % (catName, attName, catParent, attParent))
                else:
                    oL.append(" _%s:__%s:e -> _%s:__%s:e;" % (catName, attName, catParent, attParent))

                lD[(itemName, parentItemName)] = 1
                lD[(parentItemName, itemName)] = 1

            for childItemName in tD["childItems"]:
                if filterDelivery and not self.__isItemUsed(itemName=childItemName, deliveryType=deliveryType):
                    continue
                catName = CifName.categoryPart(itemName)
                attName = CifName.attributePart(itemName)
                catChild = CifName.categoryPart(childItemName)
                attChild = CifName.attributePart(childItemName)
                #
                if (itemName, childItemName) in lD:
                    continue
                if catChild not in adjacentCategories:
                    continue
                if catChild != catName:
                    oL.append(" _%s:__%s:w -> _%s:__%s:w;" % (catChild, attChild, catName, attName))
                else:
                    oL.append(" _%s:__%s:e -> _%s:__%s:e;" % (catChild, attChild, catName, attName))

                lD[(itemName, childItemName)] = 1
                lD[(childItemName, itemName)] = 1

        oL.append("}")
        return oL, numCategoriesRendered

    def makeNeighborFigure(
        self,
        categoryName,
        graphTitle=None,
        graphSubTitle=None,
        titleFormat="text",
        figFormat="svg",
        size=None,
        maxItems=20,
        filterDelivery=False,
        deliveryType="archive",
        neighborCategoryList=None,
        maxCategories=None,
        cleanup=False,
        imageFilePath=None,
    ):
        """Driver method to create diagrams of data category relationships between the input category and either all
        of its adjacent neighbors or for selected categories in 'neighborCategoryList'.

        Optionally apply filtering to categories in current use within the archive.

        maxItems       controls target maximum number of attributes in any category object depiction.
        maxCategories  limits the number of related category objects depicted.
        cleanup        True to remove 'dot' files after processing

        Output files are in SVG format, named and stored in conventional locations for this application.

        """
        logger.debug("deliveryType %r neighborCategoryList %r ", deliveryType, neighborCategoryList)
        dotList, numCategoriesRendered = self.__generateDotInstructions(
            categoryName,
            graphTitle=graphTitle,
            graphSubTitle=graphSubTitle,
            titleFormat=titleFormat,
            maxItems=maxItems,
            filterDelivery=filterDelivery,
            deliveryType=deliveryType,
            neighborCategoryList=neighborCategoryList,
            maxCategories=maxCategories,
        )
        logger.debug("deliveryType %r numCategoriesRendered %d", deliveryType, numCategoriesRendered)
        if numCategoriesRendered == 0:
            return False
        #
        if imageFilePath is None:
            dotfn = os.path.join(self.__pI.getDictCategoryImagePath(), categoryName + "_neighbors.dot")
            svgfn = os.path.join(self.__pI.getDictCategoryImagePath(), categoryName + "_neighbors.svg")
            if filterDelivery:
                dotfn = os.path.join(self.__pI.getDictCategoryImagePath(), categoryName + "_neighbors_" + deliveryType + ".dot")
                svgfn = os.path.join(self.__pI.getDictCategoryImagePath(), categoryName + "_neighbors_" + deliveryType + ".svg")
        else:
            dotfn = os.path.join(imageFilePath, categoryName + "_neighbors.dot")
            svgfn = os.path.join(imageFilePath, categoryName + "_neighbors.svg")
            if filterDelivery:
                dotfn = os.path.join(imageFilePath, categoryName + "_neighbors_" + deliveryType + ".dot")
                svgfn = os.path.join(imageFilePath, categoryName + "_neighbors_" + deliveryType + ".svg")
        #
        #
        ofh = open(dotfn, "w", encoding="utf-8")
        ofh.write("%s" % "\n".join(dotList))
        ofh.close()
        #
        if size is not None:
            cmd = self.__pathDot + ' -T%s  -Gsize="%s" %s > %s' % (figFormat, size, dotfn, svgfn)
        else:
            cmd = self.__pathDot + " -T%s %s > %s" % (figFormat, dotfn, svgfn)
        ok = os.system(cmd)
        #
        # Remove any failed image files --
        if ok != 0:
            logger.debug("status %s for %s", ok, svgfn)
            cmd = "rm -f  %s" % svgfn
            os.system(cmd)
        #
        if cleanup:
            cmd = "rm -f  %s" % dotfn
            os.system()
        #
        return ok == 0


if __name__ == "__main__":
    pass



##
# File:  SiteGeneratorExec.py
# Date: 29-Dec-2020  jdw
#
#  Execution wrapper  --  PDBx/mmCIF site generator
#
#  Updates:
#
##
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import argparse
import logging
import os
import sys

from mmcif.sitegen.wf.HtmlGeneratorWf import HtmlGeneratorWf
from mmcif.sitegen.wf.NeighborFiguresWf import NeighborFiguresWf


HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


def main():
    parser = argparse.ArgumentParser()
    #
    parser.add_argument("--web_gen_path", default=None, help="Top path to website generated content")
    parser.add_argument("--web_file_assets_path", default=None, help="Top path for website source file assests")
    parser.add_argument("--html", default=False, action="store_true", help="Generate HTML content")
    parser.add_argument("--images", default=False, action="store_true", help="Generate image content")
    parser.add_argument("--test_mode_flag", default=False, action="store_true", help="Test mode flag (default=False)")
    #
    args = parser.parse_args()
    #
    try:
        websiteGenPath = args.web_gen_path
        websiteFileAssetsPath = args.web_file_assets_path
        testModeFlag = args.test_mode_flag
        doHtml = args.html
        doImages = args.images
    except Exception as e:
        logger.exception("Argument processing problem %s", str(e))
        parser.print_help(sys.stderr)
        exit(1)
    #
    if not websiteGenPath or not websiteFileAssetsPath:
        parser.print_help(sys.stderr)
        exit(1)
    # ----------------------- - ----------------------- - ----------------------- - ----------------------- - ----------------------- -
    if doHtml:
        hgWf = HtmlGeneratorWf(websiteGenPath=websiteGenPath, websiteFileAssetsPath=websiteFileAssetsPath, testMode=testModeFlag)
        ok = hgWf.run()
        logger.info("Completed HTML generation actions with status %r", ok)

    if doImages:
        nfWf = NeighborFiguresWf(websiteGenPath=websiteGenPath, websiteFileAssetsPath=websiteFileAssetsPath, testMode=testModeFlag)
        ok = nfWf.run()
        logger.info("Completed image generation actions with status %r", ok)


if __name__ == "__main__":
    main()



##
# File:    HtmlGenerator.py
# Author:  jdw
# Date:    19-Aug-2013
# Version: 0.001
#
#  Updates:
#   10-Mar-2018 jdw Py2-P3 and refactor for Python packaging --
#   30-Dec-2020 jdw cleanup and Py39
##
"""
Classes to manage creation of files and directories representing PDBx/mmCIF
dictionaries in HTML format.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import logging
import os
import shutil
import stat

logger = logging.getLogger(__name__)


class HtmlTemplates(object):
    def __init__(self):
        #
        # These are very thin templates that rely on server-side includes for
        # the particulars.
        #
        # Top of page template
        #
        self.__templatePageHeader = """<!DOCTYPE html>
<html lang="en">
  <head>
   <!--#include virtual="/includes/head_common_bs.html"-->
    <meta name="description" content="PDBx/mmCIF Data Dictionary %s">
    <meta name="author" content="Worldwide Protein Data Bank">
    <title>%s</title>
  </head>

  <body>
    <!-- Wrap all page content here -->
    <div id="wrap">

      <!--#include virtual="/includes/page_header_bs.html"-->
      <div class="container">
    """
        #
        self.__templatePdbmlPageHeader = """<!DOCTYPE html>
<html lang="en">
  <head>
   <!--#include virtual="/includes/head_common_bs.html"-->
    <meta name="description" content="PDBx/mmCIF Resources %s">
    <meta name="author" content="Worldwide Protein Data Bank">
    <title>%s</title>
  </head>

  <body>
    <!-- Wrap all page content here -->
    <div id="wrap">

      <!--#include virtual="/includes/pdbml_page_header_bs.html"-->
      <div class="container">
    """
        #
        self.__templatePageTrailer = """
     </div> <!-- end top container-->
     </div> <!-- end wrap -->

    <!-- END TEXT HERE  -->
    <!--#include virtual="/includes/page_javascript_bs.html"-->
    <!--#include virtual="/includes/page_footer_bs.html"-->
  </body>
</html>
    """
        #

    def getPageHeader(self, title="Pdbx/mmCIF Dictionary Resources"):
        return self.__templatePageHeader % (title, title)

    def getPdbmlPageHeader(self, title="Pdbx/mmCIF Dictionary Resources"):
        return self.__templatePdbmlPageHeader % (title, title)

    def getPageTrailer(self):
        return self.__templatePageTrailer

    def getPageTitle(self, title, subTitle):
        oL = []
        oL.append('    <div class="my-page-header">')
        oL.append("      <h2>%s <small>%s</small></h2>" % (title, subTitle))
        oL.append("    </div>")
        return "\n".join(oL)

    def getTopNavbar(self, navTitle, contentTypeKey, pathInfoObj):
        #  Template for tabbed navigation within each generated page.
        #
        #  Customizable: title, subtitle, and a key to set a tab active property.
        #
        pI = pathInfoObj
        contentTypeList = pI.getContentTypeList()

        oL = []
        oL.append('    <div class="row">')
        oL.append('      <div class="col-md-1 my-nav-title"><div class="pull-right"> <h4>%s</h4></div></div>' % navTitle)
        oL.append('       <div class="col-md-11">')
        oL.append('        <ul class="nav nav-tabs">')
        for contentType in contentTypeList:
            dN = pI.getContentTypeDisplayName(contentType)
            sUrl = pI.getContentTypeIndexUrl(contentType)
            if contentType == contentTypeKey:
                oL.append('<li class="active">               <a href="%s">%s</a></li>' % (sUrl, dN))
            else:
                oL.append('<li               >               <a href="%s">%s</a></li>' % (sUrl, dN))
        oL.append("</ul>")
        oL.append("</div>")
        oL.append("</div>")
        return "\n".join(oL)

    def getTopNavbarSimple(self, contentTypeKey, pathInfoObj):
        #  Template for tabbed navigation within each generated page.
        #
        #  Customizable: title, subtitle, and a key to set a tab active property.
        #
        pI = pathInfoObj
        contentTypeList = pI.getContentTypeList()

        oL = []
        oL.append('    <div class="row">')
        # oL.append('      <h2>%s <small>%s</small></h2>' % (title,subTitle))
        oL.append('       <ul class="nav nav-tabs">')
        for contentType in contentTypeList:
            dN = pI.getContentTypeDisplayName(contentType)
            sUrl = pI.getContentTypeIndexUrl(contentType)
            if contentType == contentTypeKey:
                oL.append('<li class="active">               <a href="%s">%s</a></li>' % (sUrl, dN))
            else:
                oL.append('<li               >               <a href="%s">%s</a></li>' % (sUrl, dN))
        oL.append("</ul>")
        oL.append("</div>")
        return "\n".join(oL)


class HtmlGenerator(object):
    """HTML file and directory generator utilities."""

    def __init__(self, pathInfoObj, verbose=False):
        """"""
        self.__verbose = verbose
        self.__pI = pathInfoObj
        self.__subDirPath = self.__pI.getDictDirectoryName()
        self.__htmlTopPath = self.__pI.getHtmlTopPath()
        self.__contentTypeList = self.__pI.getContentTypeList()

    def makeDirectories(self, purge=False):
        return self.__makeDirs(self.__subDirPath, self.__htmlTopPath, purge=purge)

    def __makeDirs(self, subDirPath, htmlTopPath, purge=True):
        """internal method to make directory tree for HTML content generation."""
        try:
            pth = self.__pI.getDictContentPath()
            if purge and os.access(pth, os.F_OK):
                shutil.rmtree(pth)
            #
            for contentType in self.__contentTypeList:
                pth = self.__pI.getContentTypePath(contentType)
                if not os.access(pth, os.F_OK):
                    os.makedirs(pth, 0o755)

            pth = self.__pI.getDictCategoryImagePath()
            if not os.access(pth, os.F_OK):
                os.makedirs(pth, 0o755)

            pth = self.__pI.getDictItemImagePath()
            if not os.access(pth, os.F_OK):
                os.makedirs(pth, 0o755)
            return True
        except Exception as e:
            logger.error("HtmlGenerator.__makeDirs() failed for %s and %s", subDirPath, htmlTopPath)
            logger.exception("Failing with %s", str(e))
        return False

    def writeHtmlFile(self, contentObjName, title, subTitle, contentType, htmlContentList, navBarContentType="default"):
        """Render a standard page using common header and footer and navbar.

        Input HTML content list is inserted within the body of the page.

        """
        try:
            navBarContentSelector = contentType if navBarContentType == "default" else navBarContentType
            filePath = self.__pI.getContentTypeObjPath(contentObjName, contentType)
            logger.debug("writing file %s", filePath)

            ht = HtmlTemplates()
            ofh = open(filePath, "w", encoding="utf-8")
            pageTitle = str(title) + " " + str(subTitle)
            ofh.write("%s\n" % ht.getPageHeader(title=pageTitle))
            ofh.write("%s\n" % ht.getPageTitle(title, subTitle))
            ofh.write("%s\n" % ht.getTopNavbar("Browse:", navBarContentSelector, self.__pI))
            ofh.write("%s" % "\n".join(htmlContentList))

            ofh.write("%s\n" % ht.getPageTrailer())
            ofh.close()
            st = os.stat(filePath)
            os.chmod(filePath, st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
            return True
        except Exception as e:
            logger.error("failed for %s", filePath)
            logger.exception("Failing with %s", str(e))
        return False


if __name__ == "__main__":
    pass



##
# File:    DictionaryItemCoverage.py
# Author:  jdw
# Date:    28-Oct-2020
# Version: 0.001
#
# Updates:
##
"""
Class providing data item coverage statistics.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"

import logging
import os

from rcsb.utils.io.MarshalUtil import MarshalUtil

logger = logging.getLogger(__name__)


HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))


class DictionaryItemCoverage(object):
    """Methods providing item coverage statistics for PDB archive and chemical reference data files."""

    def __init__(self, coverageDirPath):
        #
        # Paths to data files with coverage statistics -
        #
        self.__pathArchiveItemCounts = os.path.join(coverageDirPath, "scan-pdbx-item-coverage.tdd")
        self.__pathPrdItemCounts = os.path.join(coverageDirPath, "scan-bird-item-coverage.tdd")
        self.__pathCcItemCounts = os.path.join(coverageDirPath, "scan-chem_comp-item-coverage.tdd")
        self.__pathFamilyPrdItemCounts = os.path.join(coverageDirPath, "scan-bird_family-item-coverage.tdd")
        #

    def getItemCoverage(self, deliveryType="archive"):
        """Return a dictionary of usage counts for each data item for the input delivery type.

        deliveryTypes = [ 'archive', 'cc', 'prd', 'family']
        """
        if deliveryType in ["archive"]:
            return self.__getItemCounts(self.__pathArchiveItemCounts)
        elif deliveryType in ["cc"]:
            return self.__getItemCounts(self.__pathCcItemCounts)
        elif deliveryType in ["prd", "bird"]:
            return self.__getItemCounts(self.__pathPrdItemCounts)
        elif deliveryType in ["family", "bird-family"]:
            return self.__getItemCounts(self.__pathFamilyPrdItemCounts)
        else:
            return {}

    def __getItemCounts(self, itemCoverageFilePath):
        #
        mU = MarshalUtil()
        rowList = mU.doImport(itemCoverageFilePath, fmt="tdd", rowFormat="list")
        itemCountD = {}
        for row in rowList:
            itemCountD[row[0]] = int(row[1])
        #
        return itemCountD



##
# File:    DictionaryFileUtils.py
# Author:  jdw
# Date:    19-Aug-2013
# Version: 0.001
##
"""
Utility methods for accessing dictionary files.

"""

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import logging
import os
import sys

from mmcif.api.DictionaryApi import DictionaryApi
from rcsb.utils.io.MarshalUtil import MarshalUtil

logger = logging.getLogger(__name__)


class DictionaryFileUtils(object):
    """Utility methods for accessing dictionary files."""

    def __init__(self, dictFilePath, verbose=False):
        """"""

        self.__verbose = verbose
        self.__dictFilePath = dictFilePath
        #
        #  Assign the dictionary name for the input dictionary using dictionary file path.
        #
        self.__dictDirName = self.__getDictName(dictFilePath)
        #
        # Create a reference to the dictionary API -
        #
        self.__dApi = None
        #

    def __getDictName(self, dictFilePath):
        """Extract the dictionary name from the dictioary file path."""
        try:
            (_, fN) = os.path.split(dictFilePath)
            (nm, _) = os.path.splitext(fN)
            return nm
        except Exception as e:
            logger.error("DictionaryFileUtils.__getDictName() failed for %s", dictFilePath)
            logger.exception("Failing with %s", str(e))
        return None

    def getApi(self):
        self.__dApi = self.__getApi(dictPath=self.__dictFilePath)
        return self.__dApi

    def __getApi(self, dictPath, consolidate=True, replaceDefinition=True):
        """"""
        try:
            mU = MarshalUtil()
            containerList = mU.doImport(dictPath, fmt="mmcif-dict")
            return DictionaryApi(containerList=containerList, consolidate=consolidate, replaceDefinition=replaceDefinition, verbose=self.__verbose)
        except Exception as e:
            logger.error("DictionaryFileUtils.__setup() dictionary API construction failed for %s", dictPath)
            logger.exception("Failing with %s", str(e))

    def dump(self, ofh=None):
        """Dump dictionary contents --"""
        myOut = ofh if ofh is not None else sys.stdout

        self.__dApi.dumpCategoryIndex(fh=myOut)
        self.__dApi.dumpEnumFeatures(fh=myOut)
        self.__dApi.dumpFeatures(fh=myOut)
        self.__dApi.dumpMethods(fh=myOut)
        # self.__dApi.dumpDataSections(fh=myOut)
        self.__dApi.dumpItemLinkedGroups(fh=myOut)


if __name__ == "__main__":
    pass



##
# File:    HtmlMarkupUtils.py
# Author:  jdw
# Date:    19-Aug-2013
# Version: 0.001
#
# Updates:
#  28-Dec-2020 jdw - refactor and py39 update
#
##
"""
Utility methods for creating HTML markup.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import copy
import logging
import re
from itertools import cycle

logger = logging.getLogger(__name__)


class HtmlMarkupUtils(object):
    """Utility methods for creating markup form indivdual HTML elements."""

    _htmlEscapeTable = {
        "&": "&amp;",
        '"': "&quot;",
        "'": "&apos;",
        ">": "&gt;",
        "<": "&lt;",
    }

    def __init__(self, verbose=False):
        """"""
        self.__verbose = verbose

    def htmlEscape(self, text):
        """Produce entities within text."""
        return "".join(HtmlMarkupUtils._htmlEscapeTable.get(c, c) for c in text)

    def escapeText(self, inputText, fmt="ascii", markupMath=False):
        if inputText is None:
            return ""
        if fmt == "ascii":
            inputText = self.htmlEscape(inputText)
            if markupMath:
                inputText = self.__replaceDelimiters(inputText, delimiter="~", alternateMarkup=["<sub>", "</sub>"])
                inputText = self.__replaceDelimiters(inputText, delimiter=r"\^", alternateMarkup=["<sup>", "</sup>"])
            return inputText
        else:
            return inputText

    def __replaceDelimiters(self, inputString, delimiter="~", alternateMarkup=None):
        am = alternateMarkup if alternateMarkup else ["<sub>", "</sub>"]
        aa = cycle(am)
        return re.sub(delimiter, lambda x: next(aa), inputString)

    def getFormatted(self, inputText, wrapper="pre", fmt="ascii", markupMath=False, addLeadingSpace=True):
        """"""
        if addLeadingSpace:
            lS = " "
        else:
            lS = ""

        if markupMath:
            inputText = self.escapeText(inputText, fmt="ascii")
            inputText = self.__replaceDelimiters(inputText, delimiter="~", alternateMarkup=["<sub>", "</sub>"])
            inputText = self.__replaceDelimiters(inputText, delimiter=r"\^", alternateMarkup=["<sup>", "</sup>"])
            wrapper = "verbatim"
            fmt = "html"

        if fmt == "ascii":
            if wrapper == "pre":
                html = '<pre class="pre-scrollable">%s%s</pre>' % (lS, self.escapeText(inputText))
            elif wrapper == "code":
                html = "<code>%s%s</code>" % (lS, self.escapeText(inputText))
            elif wrapper == "verbatim":
                inputText = inputText.replace("\n", "<br />")
                html = '<div class="my-monospace highlight">%s%s</div>' % (lS, self.escapeText(inputText))
            else:
                html = '<div class="my-monospace">%s%s</div>' % (lS, self.escapeText(inputText))
        else:
            if wrapper == "pre":
                html = '<pre class="pre-scrollable">%s%s</pre>' % (lS, inputText)
            elif wrapper == "code":
                html = "<code>%s%s</code>" % (lS, inputText)
            elif wrapper == "verbatim":
                # inputText = inputText.replace('\n','<br />')
                html = '<div class="my-monospace higlight">%s%s</div>' % (lS, inputText)
            else:
                html = '<div class="my-monospace highlight">%s%s</div>' % (lS, inputText)
        return html

    def getAnchor(self, contentUrl, contentLabel, cssClassString=None, fmt="ascii"):
        label = self.escapeText(inputText=contentLabel, fmt=fmt)
        if cssClassString is None:
            html = '<a href="%s">%s</a>' % (contentUrl, label)
        else:
            html = '<a href="%s" class="%s">%s</a>' % (contentUrl, cssClassString, label)
        #
        return html

    def getGlyphAnchor(self, tipText, glyphPath, cssClass="pull-right", imageCssClass=""):
        return '<a href="#" class="mytip %s" data-toggle="tooltip"  data-placement="left" title="%s"> <img class="%s" src="%s"></a>' % (cssClass, tipText, imageCssClass, glyphPath)

    def getButtonAnchor(self, contentLabel, url="#", fmt="ascii", cssClassAdd="btn-lg btn-primary", dataAttributes=""):
        label = self.escapeText(inputText=contentLabel, fmt=fmt)
        return '<a %s class="btn %s" href="%s">%s</a>' % (dataAttributes, cssClassAdd, url, label)

    def addModalButtonTrigger(self, contentLabel, fmt="ascii", modalId=None, cssClassAdd="btn-primary btn-lg"):
        """"""
        label = self.escapeText(inputText=contentLabel, fmt=fmt)
        return '<a data-toggle="modal" href="#%s" class="btn %s">%s</a>' % (modalId, cssClassAdd, label)

    def getImage(self, src, altText, width, height, cssClassAdd=""):
        return '<img src="%s" width=%s height=%s alt="%s" class="%s">' % (src, altText, width, height, cssClassAdd)

    def getBadge(self, badgeText, addClass=None):
        if addClass is not None:
            return '<span class="badge %s ">%s</span>' % (badgeText, addClass)
        else:
            return '<span class="badge">%s</span>' % badgeText

    def getIndexTitle(self, title):
        # return '<h3> - &nbsp;%s &nbsp; - </h3>' % title
        return "-&nbsp;%s &nbsp;-" % title


class HtmlComponentMarkupUtils(object):
    """Utility methods for creating markup for HTML components such as tables, panels,
    accordions, list-groups, description lists, ...

    """

    def __init__(self, verbose=False):
        """"""
        self.__verbose = verbose
        self.__oL = []
        self.__mU = HtmlMarkupUtils(verbose=self.__verbose)

    def getHtmlList(self):
        """Return a copy of the state of the generated HTML."""
        return copy.deepcopy(self.__oL)

    def clear(self):
        self.__oL = []

    def beginContainer(self, cType="row"):
        self.__oL.append('<div class="%s">' % cType)

    def endContainer(self):
        self.__oL.append("</div>")

    def addContent(self, htmlList):
        self.__oL.extend(htmlList)

    def beginPanel(self, title, style="panel-default", fmt="ascii"):
        self.__oL.append('<div class="panel %s">' % style)
        self.__oL.append('<div class="panel-heading">')
        val = self.__mU.escapeText(title, fmt=fmt)
        self.__oL.append('<h3 class="panel-title">%s</h3>' % val)
        self.__oL.append("</div>")
        self.__oL.append('<div class="panel-body">')

    def endPanel(self):
        self.__oL.append("</div>")
        self.__oL.append("</div>")

    def beginTable(self, columnNameList, style="default", fmt="ascii"):
        #
        _ = style
        # self.__curTableColumnCount = len(columnNameList)
        self.__oL.append('<div class="my-table-scrollable">')
        self.__oL.append('<table class="table table-striped table-condensed ">')
        self.__oL.append("<thead>")
        self.__oL.append("<tr>")
        for colName in columnNameList:
            val = self.__mU.escapeText(colName, fmt=fmt)
            self.__oL.append("<th>%s</th>" % val)
        self.__oL.append("</tr>")
        self.__oL.append("</thead>")
        self.__oL.append("<tbody>")

    def addTableRow(self, rowValueList, fmt="ascii", newLines="verbatim", markupMath=False):
        self.__oL.append("<tr>")
        for rowValue in rowValueList:
            val = self.__mU.escapeText(rowValue, fmt=fmt, markupMath=markupMath)

            if newLines == "verbatim":
                tV = val.replace("\n", "<br />")
                self.__oL.append('<td class="my-monospace">%s</td>' % tV)
            else:
                self.__oL.append("<td>%s</td>" % val)

        self.__oL.append("</tr>")

    def endTable(self):
        self.__oL.append("</tbody>")
        self.__oL.append("</table>")
        self.__oL.append("</div>")

    def beginDescriptionList(self, style="default"):
        if style in ["default"]:
            self.__oL.append('<dl class="dl-horizontal dl-lg">')
        else:
            self.__oL.append('<dl class="dl-lg">')

    def addDescription(self, termSt, descriptionSt, formatTerm="ascii", formatDescription="ascii", cssClassTerm=None, cssClassDescription=None, extraSpace=False):

        tVal = self.__mU.escapeText(termSt, fmt=formatTerm)
        dVal = self.__mU.escapeText(descriptionSt, fmt=formatDescription)

        if cssClassTerm is not None:
            cssT = ' class="%s" ' % cssClassTerm
        else:
            cssT = ""

        if cssClassDescription is not None:
            cssD = ' class="%s" ' % cssClassDescription
        else:
            cssD = ""

        self.__oL.append("<dt %s >%s</dt>" % (cssT, tVal))
        self.__oL.append("<dd %s >%s</dd>" % (cssD, dVal))
        if extraSpace:
            self.__oL.append("<br />")

    def endDescriptionList(self):
        self.__oL.append("</dl>")

    def beginAccordionPanelGroup(self, panelGroupId="1"):
        self.__oL.append('<div class="panel-group my-spacer" id="%s">' % panelGroupId)

    def endAccordionPanelGroup(self):
        self.__oL.append("</div> <!-- end accordion panel group -->")

    def addAccordionPanel(self, title="myPanel", subTitle=None, panelTextList=None, panelId="1", panelGroupId="1", openFlag=False, toggleText="(view/hide)", topId=None):

        op = "in" if openFlag else ""
        if topId is not None:
            self.__oL.append('<div id="%s" class="panel panel-default"> <!-- begin top accordion panel -->' % topId)
        else:
            self.__oL.append('<div class="panel panel-default"> <!-- begin top accordion panel -->')

        self.__oL.append('   <div class="panel-heading">')
        self.__oL.append('       <div class="row">')
        if subTitle is None:
            self.__oL.append('          <div class="col-md-8">')
            self.__oL.append('	             <h4 class="panel-title"> %s </h4> ' % title)
            self.__oL.append("          </div>")
            self.__oL.append('          <div class="col-md-4">')
            self.__oL.append('              <a class="accordion-toggle pull-right" data-toggle="collapse" data-parent="#%s" href="#%s">' % (panelGroupId, panelId))
            self.__oL.append(" %s" % (toggleText))
            self.__oL.append("              </a>")
            self.__oL.append("           </div>")
        else:
            self.__oL.append('          <div class="col-md-3">')
            self.__oL.append('	             <h4 class="panel-title"> %s </h4> ' % title)
            self.__oL.append("          </div>")
            self.__oL.append('          <div class="col-md-5">')
            self.__oL.append('	            <h4 class="panel-title"> %s </h4> ' % subTitle)
            self.__oL.append("          </div>")

            self.__oL.append('          <div class="col-md-4">')
            self.__oL.append('             <a class="accordion-toggle pull-right" data-toggle="collapse" data-parent="#%s" href="#%s">' % (panelGroupId, panelId))
            self.__oL.append(" %s" % (toggleText))
            self.__oL.append("             </a>")
            self.__oL.append("          </div>")
        self.__oL.append("       </div> <!-- end heading row -->")
        self.__oL.append("   </div> <!-- end panel heading -->")

        self.__oL.append('    <div id="%s" class="panel-collapse collapse %s">' % (panelId, op))
        self.__oL.append('      <div class="panel-body">')
        self.__oL.append("<!-- BEGIN inserted markup -->")
        if panelTextList:
            self.__oL.extend(panelTextList)
        self.__oL.append("<!-- END inserted markup -->")
        self.__oL.append("      </div> <!-- end panel body-->")
        self.__oL.append("    </div> <!-- end panel-collapse -->")
        self.__oL.append("</div> <!-- end top accordion panel panel-default -->")

    def beginLinkListGroup(self):
        self.__oL.append('<div class="list-group">')

    def addLinkListItem(self, linkUrl, linkValue, fmt="ascii", active=False):
        aF = "active" if active else ""
        val = self.__mU.escapeText(linkValue, fmt=fmt)
        self.__oL.append('<a href="%s" class="list-group-item %s">%s</a>' % (linkUrl, aF, val))

    def endLinkListGroup(self):
        self.__oL.append("</div>")

    def beginListGroup(self):
        self.__oL.append('<ul class="list-group">')

    def addListItem(self, listItem, fmt="ascii", active=False, cssClass=""):
        aF = "active" if active else ""
        val = self.__mU.escapeText(listItem, fmt=fmt)
        self.__oL.append('<li class="list-group-item %s %s">%s</li>' % (aF, cssClass, val))

    def endListGroup(self):
        self.__oL.append("</ul>")

    def beginInLineList(self):
        self.__oL.append('<ul class="list-inline">')

    def addInLineListItem(self, listItem, fmt="ascii", cssClass=""):
        val = self.__mU.escapeText(listItem, fmt=fmt)
        self.__oL.append('<li class="%s">%s</li>' % (cssClass, val))

    def addInLineList(self, itemList, fmt="ascii", cssClass=""):
        for listItem in itemList:
            val = self.__mU.escapeText(listItem, fmt=fmt)
            self.__oL.append('<li class="%s">%s</li>' % (cssClass, val))

    def endInLineList(self):
        self.__oL.append("</ul>")

    def addModalImageDialog(self, modalId=None, modalTitle="", imagePath="", imageText="", cssModalSect="my-image-scrollable", cssModalClose="btn-wwpdb-green"):
        """
         Template markup --
         <!-- Button trigger modal -->
         <a data-toggle="modal" href="#myModal" class="btn btn-primary btn-lg">Launch demo modal</a>
         <a data-toggle="modal" href="#myModal" class="btn btn-primary btn-lg"><img src="figure-icon.svg" width=70 height=70 alt="my image" class="img-thumbnail"></a> some related text

         <!-- Modal -->
         <div class="modal fade" id="myModal" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
           <div class="my-modal-dialog">
               <div class="modal-content">
                  <div class="modal-header">
                      <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
                      <h4 class="modal-title">Category Diagram</h4>
                  </div>
                  <div class="modal-body">
                       <div class="row my-image-scrollable">
                          <object type="image/svg+xml" data="entity_neighbors.svg" border="1"></object>
                       </div>
                  </div>
                  <div class="modal-footer">
                      <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
                  </div>
               </div><!-- /.modal-content -->
           </div><!-- /.modal-dialog -->
        </div><!-- /.modal -->
        """
        _ = imageText
        self.__oL.append('<div class="modal fade" id="%s" tabindex="-1" role="dialog" aria-labelledby="%sLabel" aria-hidden="true">' % (modalId, modalId))
        self.__oL.append('    <div class="my-modal-dialog">')
        self.__oL.append('        <div class="modal-content">')
        self.__oL.append('            <div class="modal-header">')
        self.__oL.append('                <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>')
        self.__oL.append('                         <h4 class="modal-title">%s</h4>' % modalTitle)
        self.__oL.append("            </div>")
        self.__oL.append('            <div class="modal-body">')
        self.__oL.append('                  <div class="row %s">' % cssModalSect)
        #  using object and image  tags was unsatisfactory -
        # self.__oL.append('                       <object type="image/svg+xml" data="%s" border="1"></object>' % imagePath)
        # self.__oL.append('                       <iframe class="my-image-iframe" width="100%s" height="700" src="%s"></iframe>'  % ('%',imagePath))
        self.__oL.append('                       <iframe class="my-iframe-handle" src="%s"></iframe>' % (imagePath))
        self.__oL.append("                  </div>")
        self.__oL.append("            </div>")
        self.__oL.append('            <div class="modal-footer">')
        self.__oL.append('                <button type="button" class="btn %s" data-dismiss="modal">Close</button>' % cssModalClose)
        self.__oL.append("            </div>")
        self.__oL.append("       </div><!-- /.modal-content -->")
        self.__oL.append("   </div><!-- /.modal-dialog -->")
        self.__oL.append("</div><!-- /.modal -->")


if __name__ == "__main__":
    pass



##
# File:    HtmlPathInfo.py
# Author:  jdw
# Date:    19-Aug-2013
# Version: 0.001
#
# Updates:
#
#  30-Sep-2013  jdw add paths for directories containing images -
#  28-Dec-2020  jdw cleanup and py39
##
"""
Classes to manage physical organization and path information for the HTML rendering of dictionaries.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import logging
import os

logger = logging.getLogger(__name__)


class HtmlPathInfo(object):
    def __init__(self, dictFilePath, dictDirectoryName=None, htmlDocsPath=".", htmlTopDirectoryName="dictionaries", verbose=False):
        """Manage the physical organization and path information for the HTML rendering of dictionaries."""
        self.__verbose = verbose
        self.__dictFilePath = dictFilePath
        self.__htmlDocsPath = htmlDocsPath
        self.__htmlTopDir = htmlTopDirectoryName
        #
        #  Assign the subdirectory name for the input dictionary  from dictionary file path.
        #
        self.__dictDirectoryName = self.__getDictName(dictFilePath) if dictDirectoryName is None else dictDirectoryName
        #
        # Full path to target content path - Typically HTDOCs root path plus "dictionaries"
        #
        self.__topPath = self.__setTopPath(self.__htmlDocsPath, self.__htmlTopDir)
        #
        # The HTML rendering is divided into the logical sections that are mapped to physical paths in this module.
        self.__contentTypeList = [("Index", "Dictionary"), ("Groups", "Category Groups"), ("Categories", "Data Categories"), ("Items", "Data Items"), ("Data", "Supporting Data")]
        #
        self.__contentTypeD = {}
        for v in self.__contentTypeList:
            self.__contentTypeD[v[0]] = v[1]
        #

    def getContentTypeList(self):
        """Major content types (subdirectories) within the generated HTML content."""
        return [v[0] for v in self.__contentTypeList]

    def getContentTypeDisplayName(self, contentType):
        try:
            return self.__contentTypeD[contentType]
        except Exception as e:
            logger.error("HtmlGenerator.getContentTypeDisplayName() failed for %s", contentType)
            logger.exception("Failing with %s", str(e))
            return ""

    def getDictFilePath(self):
        """Full path to target dictionary text file."""
        return self.__dictFilePath

    def getHtmlDocsPath(self):
        """HTTPD document root -"""
        return self.__htmlDocsPath

    def getHtmlTopDirectoryName(self):
        """Relative path within the HTTPD document root containing HTML content."""
        return self.__htmlTopDir

    def getHtmlTopPath(self):
        """Full path to dictectory containing HTML content."""
        return self.__topPath

    def getDictDirectoryName(self):
        return self.__dictDirectoryName

    def getContentTypeIndexUrl(self, contentType):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, contentType, "index.html")

    def getContentTypeIndexPath(self, contentType):
        return os.path.join(self.__topPath, self.__dictDirectoryName, contentType, "index.html")

    def __escapeFileName(self, name):
        """Handle limited set of cases of a file names containing problematic characters -"""
        return name.replace("/", "_over_")

    def getContentTypeObjUrl(self, contentObjName, contentType):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, contentType, self.__escapeFileName(contentObjName) + ".html")

    def getContentTypeObjPath(self, contentObjName, contentType):
        return os.path.join(self.__topPath, self.__dictDirectoryName, contentType, self.__escapeFileName(contentObjName) + ".html")

    def getContentTypePath(self, contentType):
        return os.path.join(self.__topPath, self.__dictDirectoryName, contentType)

    def getDictContentPath(self):
        return os.path.join(self.__topPath, self.__dictDirectoryName)

    def getDictCategoryImagePath(self):
        return os.path.join(self.__topPath, self.__dictDirectoryName, "Images", "Categories")

    def getDictCategoryImageDirUrl(self):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, "Images", "Categories")

    def getDictItemImagePath(self):
        return os.path.join(self.__topPath, self.__dictDirectoryName, "Images", "Items")

    def __getDictName(self, dictFilePath):
        """Extract the dictionary name from the dictioary file path."""
        try:
            (_, fN) = os.path.split(dictFilePath)
            return fN
        except Exception as e:
            logger.error("HtmlGenerator.__getDictName() failed for %s", dictFilePath)
            logger.exception("Failing with %s", str(e))
        return None

    def __setTopPath(self, htmlDocsPath, htmlTopDir):
        """Set a writeable path corresponding to the top path to contain generated html content.

        Input path corresponds to the html document root  path for the target web server.

        Returns:  Top writeable path for html generated content or None

        """
        try:
            htmlDocsPath = os.path.abspath(htmlDocsPath)
            pth = os.path.join(htmlDocsPath, htmlTopDir)
            if not os.access(pth, os.W_OK):
                os.makedirs(pth, 0o755)
            return pth

        except Exception as e:
            logger.error("HtmlGenerator.setTopPath() setting to content path failed for %s %s", htmlDocsPath, htmlTopDir)
            logger.exception("Failing with %s", str(e))
        return None

    #
    # OBSOLETE
    def getCategoryUrl(self, categoryName):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, "Categories", categoryName + ".html")

    def getGroupUrl(self, groupName):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, "Groups", groupName + ".html")

    def getItemUrl(self, itemName):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, "Items", itemName + ".html")

    def getDataUrl(self, dataName):
        return os.path.join("/", self.__htmlTopDir, self.__dictDirectoryName, "Data", dataName + ".html")



##
# File:    NeighborFiguresWf.py
# Author:  jdw
# Date:    29-Dec-2020
# Version: 0.001
#
# Updates:
#
##
"""
Workflow for generating category neighbor diagram figures.
"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import logging
import os
import time

from mmcif.sitegen.dictionary import __version__
from mmcif.sitegen.dictionary.DictionaryFileUtils import DictionaryFileUtils
from mmcif.sitegen.dictionary.DictionaryItemCoverage import DictionaryItemCoverage
from mmcif.sitegen.dictionary.DictionaryRegistry import DictionaryRegistry
from mmcif.sitegen.dictionary.HtmlGenerator import HtmlGenerator
from mmcif.sitegen.dictionary.HtmlPathInfo import HtmlPathInfo
from mmcif.sitegen.dictionary.NeighborFigures import NeighborFigures

logger = logging.getLogger(__name__)

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(HERE))
## JDW starting on path adjustments


class NeighborFiguresWf(object):
    def __init__(self, websiteGenPath="/var/www/mmcif_website_generated", websiteFileAssetsPath="/var/www/mmcif_website_file_assets", testMode=False):
        self.__verbose = True
        self.__testMode = testMode
        #
        # site path details --
        self.__pathDot = self.__findGraphvizDot()
        # Top path for generated content
        self.__webGenPath = websiteGenPath
        #
        # Source files live in website file assets path -
        self.__webFileAssetsPath = websiteFileAssetsPath
        self.__dictTopDir = "dictionaries"
        self.__pdbxResourcePath = os.path.join(self.__webFileAssetsPath, self.__dictTopDir)
        self.__coveragePath = os.path.join(self.__webFileAssetsPath, "coverage")
        self.__registryPath = os.path.join(self.__webFileAssetsPath, "config", "mmcif_dictionary_registry.json")
        #
        self.__dR = DictionaryRegistry(self.__registryPath)
        self.__dictionaryNameList = self.__dR.getDictionaryNameList()
        self.__internalDictionaryNameList = self.__dR.getInternalDictionaryNameList()
        self.__deliveryTypeL = ["archive", "cc", "prd", "family"]
        self.__itemCountD = self.__getItemCounts(self.__deliveryTypeL)
        #
        self.__fullDictionaryNameList = []
        self.__fullDictionaryNameList.extend(self.__dictionaryNameList)
        self.__fullDictionaryNameList.extend(self.__internalDictionaryNameList)
        if self.__testMode:
            self.__fullDictionaryNameList = self.__fullDictionaryNameList[:1]

        self.__startTimeD = {}

    def __logBegin(self, taskName="task"):
        self.__startTimeD[taskName] = time.time()
        logger.info("Starting %s at %s", taskName, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def __logEnd(self, taskName="task"):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", taskName, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTimeD[taskName])

    def run(self):
        """Run workflow to render of all category-level figures for current dictionary list."""
        try:
            ok = True
            for dictName in self.__fullDictionaryNameList:
                logger.info("Starting figures generation for dictionary %s", dictName)
                self.__logBegin(taskName=dictName)
                dictPath = os.path.join(self.__pdbxResourcePath, dictName + ".dic")
                pI = HtmlPathInfo(dictFilePath=dictPath, htmlDocsPath=self.__webGenPath, htmlTopDirectoryName=self.__dictTopDir, verbose=self.__verbose)
                dfu = DictionaryFileUtils(dictFilePath=dictPath, verbose=self.__verbose)
                dApi = dfu.getApi()
                self.__makeDirectories(pathInfoObj=pI, purge=False)
                categoryNameList = dApi.getCategoryList()
                figureCount = self.__makeCategoryNeighborFiguresAuto(categoryNameList=categoryNameList, dApi=dApi, pathInfoObj=pI)
                ok1 = figureCount > 0
                ok = ok1 and ok
                self.__logEnd(taskName=dictName)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __findGraphvizDot(self):
        pathDot = os.getenv("GRAPHVIZ_DOT_BINARY", default=None)
        pthList = [pathDot] if pathDot else []
        pthList.extend(["/usr/bin/dot", "/usr/local/bin/dot", "/opt/bin/dot"])
        for pth in pthList:
            if os.path.isfile(pth) and os.access(pth, os.X_OK):
                return pth
        return None

    def __getItemCounts(self, deliveryTypeL):
        itemCountD = {}
        self.__dIC = DictionaryItemCoverage(self.__coveragePath)
        for deliveryType in deliveryTypeL:
            itemCountD[deliveryType] = self.__dIC.getItemCoverage(deliveryType=deliveryType)
        for dT, cD in itemCountD.items():
            logger.debug("dT %r length %d", dT, len(cD))
        return itemCountD

    def __makeCategoryNeighborFiguresAuto(self, categoryNameList, dApi=None, pathInfoObj=None):
        """Create neighbor figures for input categories using the input dictionary api and pathInfo objects."""
        try:
            # size=".7,.7"
            size = None
            #
            nf = NeighborFigures(dictApiObj=dApi, pathInfoObj=pathInfoObj, pathDot=self.__pathDot, verbose=self.__verbose)
            for deliveryType in self.__deliveryTypeL:
                nf.setItemCounts(self.__itemCountD[deliveryType], deliveryType=deliveryType)
            #
            dictTitle = dApi.getDictionaryTitle()
            dictVersion = dApi.getDictionaryVersion()

            figureCount = 0
            for categoryName in categoryNameList:

                title = " <br/> <br/> Category Relationship Diagram for <b>%s</b> " % categoryName.upper()
                subTitle = " in dictionary %s version %s " % (dictTitle, dictVersion)
                ok = nf.makeNeighborFigure(
                    categoryName,
                    graphTitle=title,
                    graphSubTitle=subTitle,
                    titleFormat="html",
                    figFormat="svg",
                    size=size,
                    maxItems=20,
                    filterDelivery=False,
                    deliveryType="archive",
                )
                if ok:
                    figureCount += 1
                #
                title = " <br/> <br/> Abbreviated Category Relationship Diagram for <b>%s</b> " % categoryName.upper()
                subTitle = " in dictionary %s version %s <br/> including only data categories used in current PDB entries." % (dictTitle, dictVersion)
                ok = nf.makeNeighborFigure(
                    categoryName, graphTitle=title, graphSubTitle=subTitle, titleFormat="html", figFormat="svg", size=size, maxItems=20, filterDelivery=True, deliveryType="archive"
                )

                if ok:
                    figureCount += 1

                if nf.getCategoryUseCount(categoryName, deliveryType="cc") > 0:
                    title = " <br/> <br/> Abbreviated Category Relationship Diagram for <b>%s</b> " % categoryName.upper()
                    subTitle = " in dictionary %s version %s <br/> including only data categories used in the chemical reference dictionary." % (dictTitle, dictVersion)
                    ok = nf.makeNeighborFigure(
                        categoryName, graphTitle=title, graphSubTitle=subTitle, titleFormat="html", figFormat="svg", size=size, maxItems=20, filterDelivery=True, deliveryType="cc"
                    )
                    if ok:
                        figureCount += 1

                if nf.getCategoryUseCount(categoryName, deliveryType="prd") > 0:
                    title = " <br/> <br/> Abbreviated Category Relationship Diagram for <b>%s</b> " % categoryName.upper()
                    subTitle = " in dictionary %s version %s <br/> including only data categories used in the BIRD reference dictionary." % (dictTitle, dictVersion)
                    ok = nf.makeNeighborFigure(
                        categoryName, graphTitle=title, graphSubTitle=subTitle, titleFormat="html", figFormat="svg", size=size, maxItems=20, filterDelivery=True, deliveryType="prd"
                    )
                    if ok:
                        figureCount += 1

                if nf.getCategoryUseCount(categoryName, deliveryType="family") > 0:
                    title = " <br/> <br/> Abbreviated Category Relationship Diagram for <b>%s</b> " % categoryName.upper()
                    subTitle = " in dictionary %s version %s <br/> including only data categories used in the BIRD family reference dictionary." % (dictTitle, dictVersion)
                    ok = nf.makeNeighborFigure(
                        categoryName,
                        graphTitle=title,
                        graphSubTitle=subTitle,
                        titleFormat="html",
                        figFormat="svg",
                        size=size,
                        maxItems=20,
                        filterDelivery=True,
                        deliveryType="family",
                    )
                    if ok:
                        figureCount += 1

            logger.debug("%s category count %d figure count %d", dictTitle, len(categoryNameList), figureCount)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return figureCount

    def __makeDirectories(self, pathInfoObj=None, purge=True):
        """Create file system structure for HTML dictionary rendering"""
        ok = False
        try:
            hg = HtmlGenerator(pathInfoObj=pathInfoObj, verbose=self.__verbose)
            ok = hg.makeDirectories(purge=purge)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok



##
# File:    HtmlGeneratorWf.py
# Author:  jdw
# Date:    29-Dec-2020
# Version: 0.001
#
# Updates:
##
"""
Workflow methods for rendering mmCIF dictionaries in HTML
"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2,0"


import logging
import os
import stat
import time

from mmcif.sitegen.dictionary import __version__
from mmcif.sitegen.dictionary.DictionaryFileUtils import DictionaryFileUtils
from mmcif.sitegen.dictionary.DictionaryItemCoverage import DictionaryItemCoverage
from mmcif.sitegen.dictionary.DictionaryRegistry import DictionaryRegistry
from mmcif.sitegen.dictionary.HtmlContentUtils import HtmlContentUtils
from mmcif.sitegen.dictionary.HtmlGenerator import HtmlGenerator
from mmcif.sitegen.dictionary.HtmlGenerator import HtmlTemplates
from mmcif.sitegen.dictionary.HtmlMarkupUtils import HtmlComponentMarkupUtils
from mmcif.sitegen.dictionary.HtmlMarkupUtils import HtmlMarkupUtils
from mmcif.sitegen.dictionary.HtmlPathInfo import HtmlPathInfo

logger = logging.getLogger(__name__)


class HtmlGeneratorWf(object):
    def __init__(self, websiteGenPath="/var/www/mmcif_website_generated", websiteFileAssetsPath="/var/www/mmcif_website_file_assets", testMode=False):
        self.__verbose = True
        self.__testMode = testMode
        # Top path for generated content
        self.__webGenPath = websiteGenPath
        #
        # Source files live in website file assets path -
        self.__webFileAssetsPath = websiteFileAssetsPath
        self.__dictTopDir = "dictionaries"
        self.__pdbxResourcePath = os.path.join(self.__webFileAssetsPath, self.__dictTopDir)
        # self.__pdbmlResourcePath = os.path.join(self.__webFileAssetsPath, "schema")
        self.__coveragePath = os.path.join(self.__webFileAssetsPath, "coverage")
        self.__registryPath = os.path.join(self.__webFileAssetsPath, "config", "mmcif_dictionary_registry.json")
        #
        self.__dR = DictionaryRegistry(self.__registryPath)
        self.__dictionaryNameList = self.__dR.getDictionaryNameList()
        self.__internalDictionaryNameList = self.__dR.getInternalDictionaryNameList()
        self.__schemaNameList = self.__dR.getPdbmlSchemaNameList()
        #
        self.__fullDictionaryNameList = []
        self.__fullDictionaryNameList.extend(self.__dictionaryNameList)
        self.__fullDictionaryNameList.extend(self.__internalDictionaryNameList)
        #
        if self.__testMode:
            self.__fullDictionaryNameList = self.__fullDictionaryNameList[:1]
        self.__startTimeD = {}

    def __logBegin(self, taskName="task"):
        self.__startTimeD[taskName] = time.time()
        logger.info("Starting %s at %s", taskName, time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def __logEnd(self, taskName="task"):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)", taskName, time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTimeD[taskName])

    def run(self):
        """Run workflow to render dictionaries in HTML --"""
        ok = False
        try:
            ok = self.__renderDownloadList()
            for dictName in self.__fullDictionaryNameList:
                self.__logBegin(taskName=dictName)
                dictPath = os.path.join(self.__pdbxResourcePath, dictName + ".dic")
                ok = self.__makeDirectories(pathDictionary=dictPath)
                leadingGroupList = None
                if dictName == "mmcif_mdb":
                    leadingGroupList = ["mdb_group"]
                elif dictName == "mmcif_sas":
                    leadingGroupList = ["sas_group"]
                elif dictName == "mmcif_ma":
                    leadingGroupList = ["ma_group"]
                elif dictName == "mmcif_nef":
                    leadingGroupList = ["nef_group"]
                elif dictName == "mmcif_ihm":
                    leadingGroupList = ["ihm_group"]

                ok1 = self.__renderHtmlDictionary(dictionaryName=dictName, pathDictionary=dictPath, leadingGroupList=leadingGroupList)
                ok = ok1 and ok
                self.__logEnd(taskName=dictName)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __makeDirectories(self, pathDictionary):
        """Create file system structure for HTML dictionary rendering"""
        ok = False
        try:
            pI = HtmlPathInfo(dictFilePath=pathDictionary, htmlDocsPath=self.__webGenPath, htmlTopDirectoryName=self.__dictTopDir, verbose=self.__verbose)
            hg = HtmlGenerator(pathInfoObj=pI, verbose=self.__verbose)
            ok = hg.makeDirectories(purge=False)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __writeAnyFile(self, title, subTitle, filePath, htmlContentList, flavor="PDBx"):
        """Render any page using common header and footer.

        Input HTML content list is inserted within the body of the page.

        """
        try:
            if self.__verbose:
                logger.debug("writing file flavor %s path %s", flavor, filePath)
            #
            pth, _ = os.path.split(filePath)
            if not os.access(pth, os.W_OK):
                os.makedirs(pth, 0o755)
            #
            ht = HtmlTemplates()
            ofh = open(filePath, "w", encoding="utf-8")
            pageTitle = str(title) + " " + str(subTitle)
            if flavor in ["PDBx"]:
                ofh.write("%s\n" % ht.getPageHeader(title=pageTitle))
            elif flavor in ["PDBML"]:
                ofh.write("%s\n" % ht.getPdbmlPageHeader(title=pageTitle))
            else:
                ofh.write("%s\n" % ht.getPageHeader(title=pageTitle))
            ofh.write("%s\n" % ht.getPageTitle(title, subTitle))
            ofh.write("%s" % "\n".join(htmlContentList))
            ofh.write("%s\n" % ht.getPageTrailer())
            ofh.close()
            st = os.stat(filePath)
            os.chmod(filePath, st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
            return True
        except Exception as e:
            logger.error("Failed for %s", filePath)
            logger.exception("Failing with %s", str(e))
        return False

    #
    def __makePdbxDownloadPage(self, dictionaryNameList, dictionaryInfoD, dictionaryPath="/dictionaries", schemaPath="/schema"):
        """Make the download list of PDBx dictionaries"""
        mU = HtmlMarkupUtils(verbose=self.__verbose)
        html = HtmlComponentMarkupUtils(verbose=self.__verbose)
        html.clear()
        html.beginDescriptionList(style="vertical")
        for dictionaryName in dictionaryNameList:
            dList = []
            url = os.path.join(dictionaryPath, dictionaryName + ".dic", "Index")
            dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Browser  &raquo;", cssClassString=None, fmt="html"))

            url = os.path.join(dictionaryPath, "ascii", dictionaryName + ".dic")
            dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Text  &raquo;", cssClassString=None, fmt="html"))

            url = os.path.join(dictionaryPath, "ascii", dictionaryName + ".dic.gz")
            dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Text (gz)  &raquo;", cssClassString=None, fmt="html"))

            if dictionaryInfoD[dictionaryName]["schema"] is not None:
                url = os.path.join(schemaPath, dictionaryInfoD[dictionaryName]["schema"] + ".xsd")
                dList.append(mU.getAnchor(contentUrl=url, contentLabel="PDBML Schema  &raquo;", cssClassString=None, fmt="html"))
            #
            tS = dictionaryInfoD[dictionaryName]["title"]
            dS = dictionaryInfoD[dictionaryName]["description"] + "<br />" + "&nbsp;|&nbsp;".join(dList)
            html.addDescription(termSt=tS, descriptionSt=dS, formatTerm="html", formatDescription="html", extraSpace=True)

        html.endDescriptionList()
        return html.getHtmlList()

    def __makePdbmlDownloadPage(self, dictionaryNameList, dictionaryInfoD, dictionaryPath="/dictionaries", schemaPath="/schema"):
        """Make the download list of PDBML schema -"""
        mU = HtmlMarkupUtils(verbose=self.__verbose)
        html = HtmlComponentMarkupUtils(verbose=self.__verbose)
        html.clear()
        html.beginDescriptionList(style="vertical")
        for dictionaryName in dictionaryNameList:
            dList = []
            if dictionaryInfoD[dictionaryName]["schema"] is not None:
                url = os.path.join(schemaPath, dictionaryInfoD[dictionaryName]["schema"] + ".xsd")
                dList.append(mU.getAnchor(contentUrl=url, contentLabel="PDBML Schema  &raquo;", cssClassString=None, fmt="html"))

                if "dictionary" in dictionaryInfoD[dictionaryName]:
                    myName = dictionaryInfoD[dictionaryName]["dictionary"]
                else:
                    myName = dictionaryName

                url = os.path.join(dictionaryPath, myName + ".dic", "Index")
                dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Browser  &raquo;", cssClassString=None, fmt="html"))

                url = os.path.join(dictionaryPath, "ascii", myName + ".dic")
                dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Text  &raquo;", cssClassString=None, fmt="html"))

                url = os.path.join(dictionaryPath, "ascii", myName + ".dic.gz")
                dList.append(mU.getAnchor(contentUrl=url, contentLabel="Dictionary Text (gz)  &raquo;", cssClassString=None, fmt="html"))
                #
                tS = "PDBML schema for the " + dictionaryInfoD[dictionaryName]["title"]
                dS = dictionaryInfoD[dictionaryName]["description"] + "<br />" + "&nbsp;|&nbsp;".join(dList)
                html.addDescription(termSt=tS, descriptionSt=dS, formatTerm="html", formatDescription="html", extraSpace=True)

        html.endDescriptionList()
        return html.getHtmlList()

    def __renderDownloadList(self):
        """Create HTML pages for the input dictionary --"""
        ok = False
        try:
            filePath = os.path.join(self.__webGenPath, "downloads", "downloads.html")
            htmlContentList = self.__makePdbxDownloadPage(
                dictionaryNameList=self.__dictionaryNameList, dictionaryInfoD=self.__dR.get(), dictionaryPath="/dictionaries", schemaPath="/schema"
            )
            self.__writeAnyFile(title="Browse/Download ", subTitle="Dictionaries and Schema", filePath=filePath, htmlContentList=htmlContentList)
            #
            filePath = os.path.join(self.__webGenPath, "downloads", "internal-downloads.html")
            htmlContentList = self.__makePdbxDownloadPage(
                dictionaryNameList=self.__internalDictionaryNameList, dictionaryInfoD=self.__dR.get(), dictionaryPath="/dictionaries", schemaPath="/schema"
            )
            self.__writeAnyFile(title="Browse/Download ", subTitle="Internal Dictionaries and Schema", filePath=filePath, htmlContentList=htmlContentList)

            #
            filePath = os.path.join(self.__webGenPath, "downloads", "pdbml-downloads.html")
            htmlContentList = self.__makePdbmlDownloadPage(
                dictionaryNameList=self.__schemaNameList, dictionaryInfoD=self.__dR.get(), dictionaryPath="/dictionaries", schemaPath="/schema"
            )
            ok = self.__writeAnyFile(title="Browse/Download ", subTitle="PDBML Schema", filePath=filePath, htmlContentList=htmlContentList, flavor="PDBML")

        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok

    def __renderHtmlDictionary(self, dictionaryName, pathDictionary, leadingGroupList=None):
        """Create HTML pages for the input dictionary --"""
        ok = False
        try:
            pI = HtmlPathInfo(dictFilePath=pathDictionary, htmlDocsPath=self.__webGenPath, htmlTopDirectoryName=self.__dictTopDir, verbose=self.__verbose)
            hg = HtmlGenerator(pathInfoObj=pI, verbose=self.__verbose)

            dfu = DictionaryFileUtils(dictFilePath=pathDictionary, verbose=self.__verbose)

            dApi = dfu.getApi()
            hcU = HtmlContentUtils(dictApiObj=dApi, pathInfoObj=pI, verbose=self.__verbose)
            #
            #

            dIC = DictionaryItemCoverage(self.__coveragePath)
            archiveItemNameD = dIC.getItemCoverage(deliveryType="archive")
            ccItemNameD = dIC.getItemCoverage(deliveryType="cc")
            prdItemNameD = dIC.getItemCoverage(deliveryType="prd")

            hcU.setItemCounts(archiveItemNameD, deliveryType="archive")
            hcU.setItemCounts(ccItemNameD, deliveryType="cc")
            hcU.setItemCounts(prdItemNameD, deliveryType="prd")

            try:
                tS = self.__dR.getTitle(dictionaryName=dictionaryName)
                dS = self.__dR.getDescription(dictionaryName=dictionaryName)
                aS = self.__dR.getDevelopers(dictionaryName=dictionaryName)
                mS = self.__dR.getMaintainers(dictionaryName=dictionaryName)
            except Exception as e:
                logger.warning("Missing dictionary info for %s", dictionaryName)
                tS = None
                dS = None
                aS = None
                mS = None
            downloadPath = os.path.join("/", self.__dictTopDir, "ascii")
            #
            # handle history list order -
            order = "reverse"
            if dictionaryName == "mmcif_img":
                order = "forward"
            pageHtmlList = hcU.makeDictionaryIndex(downloadPath=downloadPath, dictionaryName=dictionaryName, title=tS, description=dS, authors=aS, maintainers=mS, order=order)

            subTitle = dApi.getDictionaryTitle()
            hg.writeHtmlFile("index", title="Dictionary Index", subTitle=subTitle, contentType="Index", htmlContentList=pageHtmlList)
            #
            pageHtmlList = hcU.makeCategoryGroupIndex(leadingList=leadingGroupList)
            hg.writeHtmlFile("index", title="Category Group Index", subTitle=subTitle, contentType="Groups", htmlContentList=pageHtmlList)
            groupNameList = dApi.getCategoryGroups()
            for groupName in groupNameList:
                pageHtmlList = hcU.makeCategoryGroupPage(groupName)
                hg.writeHtmlFile(groupName, title="Category Group", subTitle=groupName, contentType="Groups", htmlContentList=pageHtmlList, navBarContentType="none")

            #
            #
            pageHtmlList = hcU.makeCategoryAlphaIndex()
            hg.writeHtmlFile("index", title="Category Index", subTitle=subTitle, contentType="Categories", htmlContentList=pageHtmlList)

            pageHtmlList = hcU.makeItemCategoryAlphaIndex(openFirst=True)
            hg.writeHtmlFile("index", title="Item Index", subTitle=subTitle, contentType="Items", htmlContentList=pageHtmlList)

            pageCount = 0
            categoryNameList = dApi.getCategoryList()
            for categoryName in categoryNameList:
                pageHtmlList = hcU.makeCategoryPage(categoryName)
                hg.writeHtmlFile(categoryName, title="Data Category", subTitle=categoryName, contentType="Categories", htmlContentList=pageHtmlList, navBarContentType="none")
                pageCount += 1
                itemNameList = dApi.getItemNameList(categoryName)
                for itemName in itemNameList:
                    pageHtmlList = hcU.makeItemPage(itemName)
                    hg.writeHtmlFile(itemName, title="Data Item", subTitle=itemName, contentType="Items", htmlContentList=pageHtmlList, navBarContentType="none")
                    pageCount += 1
            logger.debug("HTML page count %d", pageCount)
            #
            pageHtmlList = hcU.makeSupportingDataIndex()
            subTitle = dApi.getDictionaryTitle()
            ok = hg.writeHtmlFile("index", title="Supporting Data", subTitle=subTitle, contentType="Data", htmlContentList=pageHtmlList)

        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return ok



# File: setup.py
# Date: 26-Mar-2020
#
# Update:
#
import re
from setuptools import setup, find_packages


packages = []
thisPackage = "mmcif.sitegen"


with open("mmcif/sitegen/dictionary/__init__.py", "r", encoding="utf-8") as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE).group(1)


# Load packages from requirements*.txt
with open("requirements.txt", "r", encoding="utf-8") as ifh:
    packagesRequired = [ln.strip() for ln in ifh.readlines()]

with open("README.md", "r", encoding="utf-8") as ifh:
    longDescription = ifh.read()

if not version:
    raise RuntimeError("Cannot find version information")

setup(
    name=thisPackage,
    version=version,
    description="mmCIF Dictionary Site Generator",
    long_description_content_type="text/markdown",
    long_description=longDescription,
    author="John Westbrook",
    author_email="john.westbrook@rcsb.org",
    url="http://mmcif.wwpdb.org",
    #
    license="Apache 2.0",
    classifiers=(
        "Development Status :: 3 - Alpha",
        # 'Development Status :: 5 - Production/Stable',
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ),
    entry_points={
        "console_scripts": [
            "site_generator_cli=mmcif.sitegen.wf.SiteGeneratorExec:main",
        ]
    },
    #
    install_requires=packagesRequired,
    packages=find_packages(exclude=["mmcif.sitegen.tests", "tests.*"]),
    package_data={
        # If any package contains *.md or *.rst ...  files, include them:
        "": ["*.md", "*.rst", "*.txt"],
    },
    #
    test_suite="mmcif.sitegen.tests",
    tests_require=["tox"],
    #
    # Not configured ...
    extras_require={
        "dev": ["check-manifest"],
        "test": ["coverage"],
    },
    # Added for
    # command_options={"build_sphinx": {"project": ("setup.py", thisPackage), "version": ("setup.py", version), "release": ("setup.py", version)}},
    zip_safe=False,
)
